% 2D wave propagation in a waveguide by the method of characteristics
% Rigid bottom is assumed
% Coded by Keiichi Ohkawa, Technical R&D Institute, JMOD
% MC Version 1.1

function MC
clear;
tstart = cputime;
eps = 1e-16;
% Input parameter
f = 20;     %frequency (Hz)
c0 = 1500;  %sound speed in water
H = 100;    %water depth (m)
zs = 50;    %source depth
zr = 50;    %reciever depth
tn = 1000;  %iteration of time step
            %maximum range is tn*delx
flag = 0;   %0:CW and Transmission loss  
            %n(>=1):nPCW modulated by hanning and animation

% Mesh size
delz = real(c0)/f/20;        %depth
delz = H/round(H/delz); 
delx = delz;                 %range
delt = min(delx,delz)/c0;    %time

% Interval
t = 0:delt:delt*tn;            %time
x = 0:delx:real(c0)*t(end);    %range
z = 0:delz:H;                  %depth

% Mesh points of source and reviver
zs = floor(zs/delz)+1;
zr = floor(zr/delz)+1;

% Initialization
Lx = length(x)+1;
Lz = length(z)+2;
p = zeros(Lx,Lz);
u = zeros(Lx,Lz);
w = zeros(Lx,Lz);

% normalization of source level
k=2*pi*f/c0;
SL = 2*sqrt(2*pi)/sqrt((k*delx)^2/(1+(k*delx)^2))/delx; % monopole
if zs == Lz-2
SL=2*SL;  % when the source is on the bottom
end
if flag ==0
S = SL*exp(2*1i*pi*f*t);
else
T = t<1/f*flag;  
S = SL*sin(2*pi*f*t).*T;
N = floor(flag/delt/f)+1;
S(1:N) = real(ifft(fft(S(1:N).*hanning(N)')));
end

% Main program
set(gca,'nextplot','replacechildren'); 
D = -c0*delt;
for n = 1:length(t)
       % Source
       p(2,zs+1) = S(n);
       % step1
       p1 = p + w;
       [p1] = cip_2d(D,delz,delt,'Lz',p1);
       % step2
       p2 = w - p;
       [p2] = cip_2d(-D,delz,delt,'Lz',p2);
       % boundary value
       p1(:,2) = p2(:,2);
       p2(:,Lz-1) = -p1(:,Lz-1);      
       w = (p1+p2)/2;
       p = (p1-p2)/2;     
       % step3
       p3 = p + u;
       [p3] = cip_2d(D,delz,delt,'Lx',p3);
       % step4
       p4 = u - p;
       [p4] = cip_2d(-D,delz,delt,'Lx',p4); 
       % cylindrical spreading
       p4(2:end-1,:) = p4(2:end-1,:)+c0*delt./((x(2:end)')*ones(1,Lz)).*u(3:end,:);
       u = (p3+p4)/2;
       p3(3:end,:) = p3(3:end,:)-c0*delt./((x(2:end)')*ones(1,Lz)).*u(3:end,:);
       u = (p3+p4)/2;
       p = (p3-p4)/2;
       if flag >= 1
       % Animation of pulse propagation;
       contourf(x(2:end),-z,(abs(p(2:Lx-1,2:Lz-1)'+eps)));xlabel('range (m)');ylabel('Depth (m)');
       getframe(gcf);
       end
end

% SPL
if flag == 0
spl = 20*log10(abs(p)); % pressure in dB
[pr] = nmode_rigid(f,c0,H,z(zs),x(end),delx,delz);
figure(1);plot(x(2:end),spl(3:Lx,zr+1),'r',x,20*log10(abs(pr(zr,:))));
ylabel('transmission loss (dB)');xlabel('Range (m)');
legend('Mehod of Characteristic','Theory(normal mode)');
end
runtime_s = cputime - tstart
end
